/**
 *
 * ====================================================================================================================================================
 *         "AppWork Utilities" License
 *         The "AppWork Utilities" will be called [The Product] from now on.
 * ====================================================================================================================================================
 *         Copyright (c) 2009-2015, AppWork GmbH <e-mail@appwork.org>
 *         Schwabacher Straße 117
 *         90763 Fürth
 *         Germany   
 * === Preamble ===
 *     This license establishes the terms under which the [The Product] Source Code & Binary files may be used, copied, modified, distributed, and/or redistributed.
 *     The intent is that the AppWork GmbH is able to provide their utilities library for free to non-commercial projects whereas commercial usage is only permitted after obtaining a commercial license.
 *     These terms apply to all files that have the [The Product] License header (IN the file), a <filename>.license or <filename>.info (like mylib.jar.info) file that contains a reference to this license.
 * 	
 * === 3rd Party Licences ===
 *     Some parts of the [The Product] use or reference 3rd party libraries and classes. These parts may have different licensing conditions. Please check the *.license and *.info files of included libraries
 *     to ensure that they are compatible to your use-case. Further more, some *.java have their own license. In this case, they have their license terms in the java file header. 	
 * 	
 * === Definition: Commercial Usage ===
 *     If anybody or any organization is generating income (directly or indirectly) by using [The Product] or if there's any commercial interest or aspect in what you are doing, we consider this as a commercial usage.
 *     If your use-case is neither strictly private nor strictly educational, it is commercial. If you are unsure whether your use-case is commercial or not, consider it as commercial or contact us.
 * === Dual Licensing ===
 * === Commercial Usage ===
 *     If you want to use [The Product] in a commercial way (see definition above), you have to obtain a paid license from AppWork GmbH.
 *     Contact AppWork for further details: <e-mail@appwork.org>
 * === Non-Commercial Usage ===
 *     If there is no commercial usage (see definition above), you may use [The Product] under the terms of the
 *     "GNU Affero General Public License" (http://www.gnu.org/licenses/agpl-3.0.en.html).
 *
 *     If the AGPL does not fit your needs, please contact us. We'll find a solution.
 * ====================================================================================================================================================
 * ==================================================================================================================================================== */
package org.appwork.swing.components;

import java.awt.Component;
import java.awt.Graphics;

import javax.swing.Icon;
import javax.swing.JRadioButton;

import org.appwork.utils.ImageProvider.ImageProvider;

public final class RadioBoxIcon implements Icon {
    public static final RadioBoxIcon FALSE     = new RadioBoxIcon(false);
    public static final RadioBoxIcon TRUE      = new RadioBoxIcon(true);
    public static final RadioBoxIcon UNDEFINED = new RadioBoxIcon(true, false);
    private JRadioButton             cb;
    private Icon                     internalIcon;
    private int                      size;

    public RadioBoxIcon(final boolean selected, final boolean enabled) {

        cb = new JRadioButton() {
            {
                setSelected(true);
            }

            @Override
            public int getWidth() {
                return getPreferredSize().width;
            }

            @Override
            public int getHeight() {
                return getPreferredSize().height;
            }

            @Override
            public boolean isVisible() {
                return true;
            }
        };

        ;
        cb.setSelected(selected);
        // we need this workaround.
        // if we would use cb.paint(g); for every paintIcon call, this might habe sideeffects on the LAF painter.

        size = 16;

        internalIcon = ImageProvider.toImageIcon(this);

        if (!enabled) {
            internalIcon = org.appwork.resources.AWUTheme.I().getDisabledIcon(internalIcon);
        }

    }

    public RadioBoxIcon(final boolean selected) {
        this(selected, true);
    }

    @Override
    public void paintIcon(final Component c, Graphics g, final int x, final int y) {
        if (internalIcon != null) {
            internalIcon.paintIcon(c, g, x, y);

            return;
        }
        // g.setColor(Color.RED);
        // g.drawRect(0, 0, 14, 14);
        g = g.create(x, y, size, size);
        // g.translate(x, y);
        g.translate(-4, -4);
        cb.paint(g);
        g.dispose();
        // g.translate(4, 4);
        // g.translate(-x, -y);

        // g.dispose();
        // g.translate(0, -10);

    }

    @Override
    public int getIconWidth() {
        return size;
    }

    @Override
    public int getIconHeight() {
        return size;
    }
}